﻿-- Average Combat Mana ------------------------------------------------------

-- localise global variables for faster access ------------------------------
local type = type
local UnitMaxPower = UnitMaxPower
local UnitPower = UnitPower
local GetAddOnMetadata = GetAddOnMetadata
local tonumber = tonumber
local tostring = tostring
local date = date
local SendChatMessge = SendChatMessage
local tostring = tostring
local format = format

-- declare addon ------------------------------------------------------------
local LibStub = LibStub

local ACM = LibStub("AceAddon-3.0"):NewAddon("AverageCombatMana", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("AverageCombatMana", true)

-- get version from .toc - set to Development if no version
local version = GetAddOnMetadata("AverageCombatMana", "Version")
--@alpha@
if version:match("@") then
	local ACM_Revision = tonumber(("$Revision: 19 $"):match("(%d+)")) + 1
	version = "Development r"..tostring(ACM_Revision)
else
	version = "Alpha "..version
end
--@end-alpha@

-- add localisation to addon
ACM.L = L

-- declare the database
local db

-- local variables ----------------------------------------------------------
local allManaValues = {} -- stores all values during combat
local ManaDisplay = CreateFrame("Frame", "ManaDisplay", UIParent)

-- additional libraries -----------------------------------------------------
local DataBroker = LibStub("LibDataBroker-1.1")
-- local Graph = LibStub("LibGraph-2.0")
-- local Media = LibStub("LibSharedMedia-3.0")

local defaults = {
	--[===[@debug@
	char = {
		logs = {}
	},
	--@end-debug@]===]
	profile = {
		addonEnabled = true,
		chatOutput = "1-self",
		showBar = true,
		-- graphBorder = "Blizzard Dialog Gold",
		-- graphTexture = "Blizzard",		
		-- displayGraph = true,		
	}
}

function ACM:OnInitialize()
	-- register saved variables with AceDB
	db = LibStub("AceDB-3.0"):New("AverageCombatManaDB", defaults, "Default")
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db = db
	self:SetEnabledState(self.db.profile.addonEnabled)
	
	-- addon options table	
	self.options = self:OptionsTable() -- see AverageCombatManaOptions.lua
	-- add the 'Profiles' section
	self.options.args.profilesTab = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profilesTab.order = 50

	-- Register your options with AceConfigRegistry
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ACM", self.options)

	-- Add your options to the Blizz options window using AceConfigDialog
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ACM", L["Average Combat Mana"])	

	-- support for LibAboutPanel	
	if LibStub:GetLibrary("LibAboutPanel", true) then
		self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("Average Combat Mana", "Average Combat Mana")
	end
		
	-- create the frame
	ManaDisplay = ManaDisplay or CreateFrame("Frame", "ManaDisplay", UIParent)
	ManaDisplay:SetPoint(self.db.profile.barPoint or "CENTER", self.db.profile.barX or 0, self.db.profile.barY or 250)
	ManaDisplay:SetHeight(20)
	ManaDisplay:SetWidth(210)
	
	ManaDisplay:SetBackdrop({
		bgFile = "Interface\\BUTTONS\\WHITE8X8", tile = false, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = { left = 4, right = 4, top = 4, bottom = 4 },
	})
	
	ManaDisplay:SetBackdropColor(0, 0, 0, 1)
	ManaDisplay:SetBackdropBorderColor( )
	
	ManaDisplay:SetAlpha(1)
	ManaDisplay.segments = ManaDisplay.segments or {}
	
	local TEXTURE = "Interface\\BUTTONS\\WHITE8X8"
	
	for i = 1, 21 do
		local t = ManaDisplay.segments[i] or ManaDisplay:CreateTexture(nil, "ARTWORK")
		t:SetPoint("LEFT", ManaDisplay.segments[i - 1], "RIGHT", 0, 0)
		t:SetWidth(1)
		t:SetHeight(ManaDisplay:GetHeight() - 10)
		t:SetTexture(TEXTURE)
		t:SetAlpha(0.8)
		ManaDisplay.segments[i] = t
	end
	
	-- mana loss
	local k = 1
	for i = 1, 9 do		
		ManaDisplay.segments[i]:SetVertexColor(k, 0, 0)
		k = k - 0.1
	end
	
	-- neither gain nor loss
	ManaDisplay.segments[10]:SetVertexColor(0, 0, 0.5)
	ManaDisplay.segments[11]:SetVertexColor(0, 0, 1)
	ManaDisplay.segments[12]:SetVertexColor(0, 0, 0.5)
	
	-- mana gain
	k = 0.1	
	for i = 13, 21 do
		ManaDisplay.segments[i]:SetVertexColor(0, k, 0)
		k = k + 0.1
	end
	
	ManaDisplay.message = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
	ManaDisplay.message:SetPoint("BOTTOM", ManaDisplay, "TOP", 0, 5)
	ManaDisplay.message:SetText("Average Combat Mana")

	ManaDisplay.leftSide = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	ManaDisplay.leftSide:SetPoint("TOPLEFT", ManaDisplay, "BOTTOMLEFT", 5, -5)
	ManaDisplay.leftSide:SetText(L["Full Burn!"])
	ManaDisplay.leftSide:SetTextColor(1, 0, 0, 1)

	ManaDisplay.rightSide = ManaDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	ManaDisplay.rightSide:SetPoint("TOPRIGHT", ManaDisplay, "BOTTOMRIGHT", -5, -5)
	ManaDisplay.rightSide:SetText(L["Gaining Mana!"])
	ManaDisplay.rightSide:SetTextColor(0, 1, 0, 1)

	ManaDisplay.indicator = ManaDisplay:CreateTexture(nil, "OVERLAY")
	ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[11], "RIGHT")
	ManaDisplay.indicator:SetWidth(10)
	ManaDisplay.indicator:SetHeight(ManaDisplay:GetHeight() + 10)
	ManaDisplay.indicator:SetTexture("Interface\\WorldStateFrame\\WorldState-CaptureBar")
	ManaDisplay.indicator:SetTexCoord(0.8, 0.8, 0, 0.3)
	ManaDisplay.indicator:SetDesaturated(true)

	ManaDisplay:EnableMouse(true)
	ManaDisplay:SetMovable(true)
	ManaDisplay:SetClampedToScreen(true)
	ManaDisplay:RegisterForDrag("LeftButton")
	ManaDisplay:SetUserPlaced(false)
	ManaDisplay:SetScript("OnDragStart", function(self)
		if IsAltKeyDown() then
			ManaDisplay:StartMoving()
		end
	end)
	ManaDisplay:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		self:SetUserPlaced(false)
		local point, _, _, x, y = self:GetPoint(1)
		ACM.db.profile.barPoint = point
		ACM.db.profile.barX = x
		ACM.db.profile.barY = y
	end)
	
	--[[
	-- create real time graph
	self.RealTimeGraph = Graph:CreateGraphRealtime("TestRealtimeGraph",UIParent,"CENTER","CENTER",-90,90,150,150)
	self.RealTimeGraph:SetAutoScale(true)
	self.RealTimeGraph:SetGridSpacing(1, 10)
	]]--	

	-- add console commands
	self:RegisterChatCommand("acm", "SlashHandler")
	self:RegisterChatCommand("averagecombatmana", "SlashHandler")	
	
	-- create DataBroker Launcher
	if DataBroker then
		local launcher = DataBroker:NewDataObject("ACM", {
			type = "launcher",
			icon = "Interface\\Icons\\INV_Potion_168",
			OnClick = function(clickedframe, button)
				if button == "LeftButton" then
					-- keep our options table in sync with the ldb object state
					--[[
					self.db.profile.displayGraph = not self.db.profile.displayGraph
					if self.db.profile.displayGraph then
						-- display graph code here
					else
						-- hide graph code here
					end
					LibStub("AceConfigRegistry-3.0"):NotifyChange("ACM")
					]]--
				elseif button == "RightButton" then
					_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
				--[[
				elseif button == "MiddleButton" then
					self:StartTest()
				]]--
				end
			end,
			OnTooltipShow = function(self)
			GameTooltip:AddLine("Average Combat Mana".." "..version, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
			-- GameTooltip:AddLine(L["Left click to display real time graph."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			-- GameTooltip:AddLine(L["Middle click to run test display."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			GameTooltip:AddLine(L["Right click for configuration."], NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
			GameTooltip:Show()			
		end})
		self.launcher = launcher
	end
	
	-- get the poor alpha users' logs under control
	-- keep the last 20 entries just in case someone wants them
	--@alpha@
	while #self.db.char.logs >= 21 do
		table.remove(self.db.char.logs, 1)
	end
	--@end-alpha@
end

function ACM:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "NoCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "Combat")
	-- Media.RegisterCallback(self, "OnValueChanged", "UpdateMedia")
end

function ACM:OnDisable()
	self:UnregisterAllEvents()
	ManaDisplay:Hide()
	-- self.RealTimeGraph:ResetData()
	-- Media.UnregisterAllCallbacks(self)
end

function ACM:OnProfileChanged()
	db = self.db
end

function ACM:SlashHandler(input)
	--[===[@debug@
	if input == "test" then
		self:StartTest()
	end
	--@end-debug@]===]
	_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

--[[
function ACM:UpdateMedia(callback, type, handle)
	-- local fontStyle
	if type == "border" then 
		ManaDisplay:SetBackdrop({ 
			edgeFile = Media:Fetch("border", self.db.profile.barBorder), 
			tile = false, 
			edgeSize = 16, 
			insets = { left = 4, right = 4, top = 4, bottom = 4 }, 
		})
	elseif type == "font" then
		if self.db.profile.fontEffects == "NONE" then
			fontStyle = nil
		end
		ManaDisplay.message:SetFont(profile.messageFont, 18, self.db.profile.fontEffects) 
		ManaDisplay.leftSide:SetFont(profile.manaFont, 14, self.db.profile.fontEffects) 
		ManaDisplay.rightSide:SetFont(profile.manaFont, 14, self.db.profile.fontEffects)
	end 
end
]]--

local in_combat
function ACM:NoCombat()
	in_combat = false
	ManaDisplay:SetScript("OnUpdate", nil)
	
	if self.db.profile.showBar then
		if not ManaDisplay:IsShown() then
			ManaDisplay:Show()
			self.db.profile.showBar = true
		end
	else
		if ManaDisplay:IsShown() then
			ManaDisplay:Hide()
			self.db.profile.showBar = false
		end
	end
	
	-- reset indicator to posistion 0, which is 11, or centre
	ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[11], "RIGHT")
	--[===[@debug@
	ManaDisplay.leftSide:SetText(L["Full Burn!"])
	ManaDisplay.rightSide:SetText(L["Gaining Mana!"])
	--@end-debug@]===]
	--@non-debug@
	ManaDisplay.leftSide:SetText("")
	ManaDisplay.rightSide:SetText("")
	--@end-non-debug@
		
	-- calculate overall average gain or loss of Mana
	local allManaSum = 0
	
	local t = allManaValues
	for k = 1, #t do
		local v = t[k]
		allManaSum = allManaSum + v
	end
	
	-- get the average of all gains and losses for entire fight, rounding off any decimals
	local function round(combatAverage, idp)
    local mult = 10^(idp or 0)
		if combatAverage >= 0 then return math.floor(combatAverage * mult + 0.5) / mult
		else return math.ceil(combatAverage * mult - 0.5) / mult end
	end

	local combatAverage = round((allManaSum / #t))
	
	local channel = self.db.profile.chatOutput:upper()
	if channel == "2-SMARTGROUP" then
		if IsInRaid() then
			channel = "RAID"
		elseif IsInParty() then
			channel = "PARTY"
		end
	end	
	--[===[@debug@
	self:Print("channel is", channel)
	--@end-debug@]===]
		
	if channel == "1-SELF" then
		self:Print(L["Average Mana gain or loss for the entire fight was"], combatAverage)
	elseif channel ~= "0-NONE" then
		SendChatMessage(L["Average Mana gain or loss for the entire fight was"].. format(" %d", combatAverage), channel, nil, nil)
	end
	
	combatAverage = nil	
	allManaValues = {}
end

function ACM:Combat()	
	if #self.db.char.logs > 20 then
		table.remove(self.db.char.logs, 1) -- keep log file to a max of 20 entries, removing oldest
	end
	in_combat = true
	local firstManaValue = UnitPower("player", SPELL_POWER_MANA)
	
	-- must show the display in combat, or nothing happens, including calculating/printing overall average
	if not ManaDisplay:IsShown() then
		ManaDisplay:Show()
	end
	ManaDisplay.leftSide:SetText("")
	ManaDisplay.rightSide:SetText("")
	
	local counter = 0
	ManaDisplay:SetScript("OnUpdate", function(self, elapsed) 
		counter = counter + elapsed 
		if counter < 1 then return end
		counter = 0

		local currentMana = UnitPower("player", SPELL_POWER_MANA)
		
		local deltaMana = currentMana - firstManaValue or currentMana
		firstManaValue = currentMana

		local numManaValues = #allManaValues + 1
		allManaValues[numManaValues] = deltaMana
		
		-- get 10 (or less, if there are less) recent values
		local count = numManaValues >= 10 and 10 or numManaValues
		for index = numManaValues - count + 1, numManaValues - 1 do
			deltaMana = deltaMana + allManaValues[index]
		end
		
		-- average the values, dropping decimal places
		local function round(average, idp)
			local mult = 10 ^ (idp or 0)
			return floor(average * mult + 0.5) / mult
		end
		local function round2(pointer, idp)
			local mult = 10 ^ (idp or 0)
			return floor(pointer * mult + 0.5) / mult
		end

		local average = deltaMana / count
		local pointer = average
				
		while (pointer >= 11) or (pointer <= -11) do
			pointer = pointer / 10
		end		
		
		average = round(average)
		pointer = round2(pointer)
		
		if pointer < 0 then
			pointer = pointer * -1
		else
			pointer = pointer + 11
		end
		
		ManaDisplay.indicator:SetPoint("CENTER", ManaDisplay.segments[pointer], "RIGHT")
		
		if average < 0 then
			ManaDisplay.leftSide:SetFormattedText(L["Full Burn!"] .."\n%d", average)
			ManaDisplay.rightSide:SetText("")
		elseif average > 1 then
			ManaDisplay.rightSide:SetFormattedText(L["Gaining Mana!"] .."\n%d", average)
			ManaDisplay.leftSide:SetText("")
		else
			ManaDisplay.leftSide:SetText("")
			ManaDisplay.rightSide:SetText("")
		end
	end)
	
	--[===[@debug@
	self.db.char.logs[#self.db.char.logs + 1] = {
		date = date(),
		data = allManaValues,
	}
	--@end-debug@]===]
end

function ACM:ShowBar()
	if not ManaDisplay:IsShown() then
		ManaDisplay:Show()
	end
end

function ACM:HideBar()
	if ManaDisplay:IsShown() then
		ManaDisplay:Hide()
	end
end